from datetime import date
try:
    from __buildnumber__ import BUILD_NUMBER
except (ImportError, ValueError) as e:
    BUILD_NUMBER = 0
    DEVELOP = True
from __version__ import MAJOR, MINOR, RELEASE


VERSION = (MAJOR, MINOR, RELEASE, BUILD_NUMBER)


def version_info():
    return dict(
        fileversion='.'.join(str(x) for x in VERSION),
        version='.'.join(str(x) for x in VERSION[:3]),
        build_number=BUILD_NUMBER,
        year=date.today().year
    )



# coding: utf-8
from __future__ import absolute_import
import argparse
import os
from .empacotar import (
    INNOSERV, PACOTE, VERSAO, MENSAGEM, DEVELOP, empacotar
)


parser = argparse.ArgumentParser(
    description=(
        "Gera um pacote do Colibri Master. "
        "Este programa usa partes do 7-Zip,"
        " que é licenciado sob GNU LGPL."
        " O código pode ser obtido em http://www.7-zip.org"
    )
)
parser.add_argument('caminho', type=str,
                    help='Caminho dos arquivos do pacote')
parser.add_argument('--pasta_saida', dest='pasta_saida',
                    help=u'Pasta de saída')
parser.add_argument('--senha', dest='senha', default='', help='Senha')
parser.add_argument('--' + INNOSERV, dest=INNOSERV,
                    help=u'parametro IGNORADO')
parser.add_argument('--' + PACOTE, dest=PACOTE, help=u'Id do pacote')
parser.add_argument('--' + VERSAO, dest=VERSAO, help=u'Versão do produto')
parser.add_argument('--nome_exibicao', dest='nome_exibicao',
                    help=u'Nome de exibição do produto')
parser.add_argument('--nivel_sdk', dest='nivel_sdk',
                    help=u'Nivel do SDK')
parser.add_argument(
    '--' + MENSAGEM, dest=MENSAGEM,
    help=u'Mensagem descritiva do pacotes'
)
parser.register(
    'type', 'bool', lambda v: v.lower() in ("true", "t", "1")
)
parser.add_argument(
    '--' + DEVELOP, dest=DEVELOP, help=u'Pacote de desenvolvimento',
    type='bool', default=True
)
parser.add_argument(
    '--versao_base', nargs=2, dest='versoes_bases', action='append',
    help='Nome do schema e versao do banco', metavar=('schema', 'versao')
)

args = vars(parser.parse_args())

caminho = args.pop('caminho')
caminho_destino = args.pop('pasta_saida') or os.getcwd()
saida = empacotar(caminho, caminho_destino, args.pop('senha'),
                  **{k: args[k] for k in args if args[k] is not None})



import unittest
from colibri_packaging.empacotar import obter_arquivos


class TestesObterListaArquivos(unittest.TestCase):
    def _criar_arquivo_server(self, pattern):
        return dict(
            params_serv="/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART /SERVIDOR",
            destino="server", _pattern_nome=pattern
        )

    def _criar_arquivo_client(self, pattern):
        return dict(
            params_client="/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART",
            destino="client", _pattern_nome=pattern
        )

    def test_sem_pattern(self):
        # Arrange
        pasta = r"c:\temp"
        configs = dict(
            arquivos=[
                dict(
                    params_serv="/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART /SERVIDOR",
                    destino="server", nome="nomecravado.exe"
                ),
                dict(
                    params_client="/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART",
                    destino="client", nome="nome_client.exe"
                )
            ]
        )
        diretorio = [
            "nomecravado.exe",
            "nome_client.exe"
        ]
        # Act
        lista_zip = obter_arquivos(pasta, configs, diretorio)
        # Assert
        self.assertEqual(
            lista_zip, [
            'c:\\temp\\manifesto.dat',
            'c:\\temp\\nomecravado.exe',
            'c:\\temp\\nome_client.exe'
            ]
        )
        self.assertEqual(
            configs['arquivos'],
            [
                {
                    'destino': 'server', 'nome': 'nomecravado.exe',
                    'params_serv': '/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART /SERVIDOR'
                },
                {
                    'destino': 'client', 'nome': 'nome_client.exe',
                    'params_client': '/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART'
                }]
        )

    def test_sem_pattern_cade_o_arquivo(self):
        # Arrange
        pasta = r"c:\temp"
        configs = dict(
            arquivos=[
                dict(
                    params_serv="/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART /SERVIDOR",
                    destino="server", nome="nomecravado.exe"
                ),
                dict(
                    params_client="/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART",
                    destino="client", nome="nome_client.exe"
                )
            ]
        )
        diretorio = [
            "nomecravado.exe"
        ]
        # Act & Assert
        with self.assertRaisesRegex(RuntimeError, "Nome arquivo não foi encontrado: nome_client.exe"):
            lista_zip = obter_arquivos(pasta, configs, diretorio)

    def test_obter_arquivos_dupla_referencia(self):
        # Arrange
        pasta = r"c:\temp"
        configs = dict(
            arquivos=[
                self._criar_arquivo_server(".*teste\\.exe\\b"),
                self._criar_arquivo_client(".*teste\\.exe\\b")
            ]
        )
        diretorio = [
            "manifesto.dat",
            "teste.exe"
        ]
        # Act
        lista_zip = obter_arquivos(pasta, configs, diretorio)
        # Assert
        self.assertEqual(lista_zip, [r'c:\temp\manifesto.dat', r'c:\temp\teste.exe'])
        self.assertEqual(
            configs['arquivos'],
            [
                {
                    'destino': 'server', 'nome': 'teste.exe',
                    'params_serv': '/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART /SERVIDOR'
                },
                {
                    'destino': 'client', 'nome': 'teste.exe',
                    'params_client': '/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART'
                }]
        )

    def test_scripts_automagicamente(self):
        # Arrange
        pasta = r"c:\temp"
        configs = dict(
            arquivos=[
                self._criar_arquivo_server(".*teste\\.exe\\b"),
                self._criar_arquivo_client(".*teste\\.exe\\b")
            ]
        )
        diretorio = [
            "manifesto.dat",
            "teste.exe",
            "_scripts.zip"
        ]
        # Act
        lista_zip = obter_arquivos(pasta, configs, diretorio)
        # Assert
        self.assertEqual(lista_zip, [r'c:\temp\manifesto.dat', r'c:\temp\teste.exe', r'c:\temp\_scripts.zip'])
        self.assertEqual(
            configs['arquivos'],
            [
                {'destino': 'scripts', 'nome': '_scripts.zip'},
                {
                    'destino': 'server', 'nome': 'teste.exe',
                    'params_serv': '/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART /SERVIDOR'
                },
                {
                    'destino': 'client', 'nome': 'teste.exe',
                    'params_client': '/SP- /SILENT /VERYSILENT /SUPPRESSMSGBOXES /NOCANCEL /NORESTART'
                }]
        )

    def test_exe_dando_bobeira(self):
        # Arrange
        pasta = r"c:\temp"
        configs = dict(
            arquivos=[
                self._criar_arquivo_server(".*qualquer\\.exe\\b"),
                self._criar_arquivo_client(".*nao_importa\\.exe\\b")
            ]
        )
        diretorio = [
            "manifesto.dat",
            "qualquer.exe",
            "ignoto.exe"
        ]
        # Act & Assert
        with self.assertRaisesRegex(RuntimeError,
                "Arquivo executável desconhecido na pasta de empacotamento: ignoto.exe"):
            obter_arquivos(pasta, configs, diretorio)

    def test_pattern_nao_identificado(self):
        # Arrange
        pasta = r"c:\temp"
        configs = dict(
            arquivos=[
                self._criar_arquivo_server(".*qualquer\\.exe\\b"),
                self._criar_arquivo_client(".*teste\\.exe\\b")
            ]
        )
        diretorio = [
            "manifesto.dat",
            "teste.exe"
        ]
        # Act & Assert
        with self.assertRaisesRegex(RuntimeError,
                r"Pattern de nome arquivo não foi encontrado: \.\*qualquer\\\.exe\\b"):
            obter_arquivos(pasta, configs, diretorio)

    def test_deve_manter_a_ordem_pelo_tipo(self):
        # Arrange
        pasta = r"c:\temp"
        configs = dict(
            arquivos=[
                self._criar_arquivo_server(".*primeiro_server\\.exe\\b"),
                self._criar_arquivo_client(".*primeiro_client\\.exe\\b"),
                self._criar_arquivo_server(".*segundo_server\\.exe\\b"),
                self._criar_arquivo_client(".*segundo_client\\.exe\\b"),
                self._criar_arquivo_server(".*terceiro_server\\.exe\\b"),
                self._criar_arquivo_client(".*terceiro_client\\.exe\\b"),
            ]
        )
        diretorio = [
            "terceiro_server.exe",
            "primeiro_server.exe",
            "manifesto.dat",
            "segundo_client.exe",
            "terceiro_client.exe",
            "segundo_server.exe",
            "primeiro_client.exe",
        ]
        # Act
        lista_zip = obter_arquivos(pasta, configs, diretorio)

        # Assert
        self.assertEqual(
            lista_zip,
            [
                "c:\\temp\\manifesto.dat",
                "c:\\temp\\terceiro_server.exe",
                "c:\\temp\\primeiro_server.exe",
                "c:\\temp\\segundo_client.exe",
                "c:\\temp\\terceiro_client.exe",
                "c:\\temp\\segundo_server.exe",
                "c:\\temp\\primeiro_client.exe",
            ]
        )
        nomes_arquivos = [a['nome'] for a in configs['arquivos']]
        self.assertEqual(
            nomes_arquivos,
            [
                "primeiro_server.exe",
                "segundo_server.exe",
                "terceiro_server.exe",
                "primeiro_client.exe",
                "segundo_client.exe",
                "terceiro_client.exe",
            ]
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
import json
import logging
import os
import subprocess
import re
import shutil
from functools import cmp_to_key
try:
    from scripts.versao_banco import obter_versao_db_scripts
except ImportError:
    obter_versao_db_scripts = None

CAM_7ZA = os.path.join(os.path.split(__file__)[0] or os.getcwd(), '7za.exe')
MANIFESTO_SERVER = 'manifesto.server'
MANIFESTO_LOCAL = 'manifesto.local'
MANIFESTO = 'manifesto.dat'
EXTENSAO_PACOTE = '.cmpkg'
RE_ARQ_SCRIPT = re.compile(r'_scripts(\d{0,2})(\S+)?\.zip')
EXTENSAO_ARQ_SCRIPT = '.zip'
PACOTE = 'nome'
VERSAO = 'versao'
SCHEMA = 'schema'
ARQUIVOS = 'arquivos'
NOME_ARQ = 'nome'
MENSAGEM = 'mensagem'
DESTINO = 'destino'
DEVELOP = 'develop'
INNOSERV = 'innoserv'
BASES_COMPATIVEIS = 'versoes_bases'
ARQ_PACOTE = 'pacote'
ARQ_SCRIPTS = 'scripts'
ARQ_CLIENT = 'client'
ARQ_SERVIDOR = 'server'
ARQ_SHARED = 'shared'
PASTA_QA = r'd:\Builder\QA'


logger = logging.getLogger('empacotar')

DEFAULTS_PACOTE = dict(
    versao="1.0.0.0"
)
_ordem_arquivo = dict()


def _acha_ordem(arq):
    def _obter_sequencia():
        def_ret = _ordem_arquivo.get((arq['nome'], arq["destino"]), 0)
        if arq['destino'] == ARQ_SCRIPTS:
            match = RE_ARQ_SCRIPT.match(arq['nome'])
            if match and match.group(1):
                def_ret = int(match.group(1))
        return def_ret

    ordem = {
        ARQ_PACOTE: -1000,
        ARQ_SCRIPTS: 0,
        ARQ_SHARED: 1000,
        ARQ_SERVIDOR: 2000,
        ARQ_CLIENT: 3000
    }

    return ordem.get(arq['destino']) + _obter_sequencia()


def _acha_tipo(arqorg):
    arqorg = arqorg.lower()
    if RE_ARQ_SCRIPT.match(arqorg):
        return ARQ_SCRIPTS


def _obter_chaves_arquivo(dict_chaves):
    return {chave: valor for chave, valor in dict_chaves.items() if
            chave != 'nome' and not chave.startswith('_')}


def _excluir_com_prefixo(pasta, prefixo):
    for arq in os.listdir(pasta):
        if arq.endswith(EXTENSAO_PACOTE) and arq.startswith(prefixo):
            try:
                os.unlink(os.path.join(pasta, arq))
            except Exception:
                logger.exception('Falha ao remover cmpkg anterior: ' + arq)


def obter_arquivos(pasta, manifesto_dat, lista_no_diretorio):
    lista_zip = list((os.path.join(pasta, MANIFESTO),))
    lista_anterior = manifesto_dat.get(ARQUIVOS, list())

    manifesto_dat[ARQUIVOS] = list()

    def criar_arq_(arq, chaves=None):
        dictarq = dict(
            nome=arq
        )

        if chaves is not None:
            dictarq.update(_obter_chaves_arquivo(chaves))

        if dictarq.get('destino') is None:
            dictarq['destino'] = _acha_tipo(arq)

        if dictarq.get('destino'):
            manifesto_dat[ARQUIVOS].append(dictarq)
            completo = os.path.join(pasta, arq)
            if completo not in lista_zip:
                lista_zip.append(completo)
        elif arq.lower().endswith('.exe'):
            raise RuntimeError(
                f"Arquivo executável desconhecido na pasta de empacotamento: {arq}"
            )
        return dictarq

    def temos_um_match(arq, a):
        return ('_pattern_nome' in a and re.match(a['_pattern_nome'], arq)) or \
               ('nome' in a and a['nome'].lower() == arq.lower())

    for arq in lista_no_diretorio:
        if arq.lower() == MANIFESTO:
            continue

        criado = False
        # se eu encontrar a definição do arquivo na lista anterior,
        # uso o que eu encontrar
        for pos, a in enumerate(lista_anterior):
            # arquivo está marcado com pattern? Adiciono
            if temos_um_match(arq, a):
                if "__count" in a:
                    a["__count"] += 1
                else:
                    a["__count"] = 1
                dictarq = criar_arq_(arq, _obter_chaves_arquivo(a))
                _ordem_arquivo[(arq, dictarq['destino'])] = pos
                criado = True

        if not criado:
            dictarq = criar_arq_(arq)
            _ordem_arquivo[(arq, dictarq['destino'])] = 0

    for a in lista_anterior:
        if '_pattern_nome' in a and "__count" not in a:
            raise RuntimeError(
                f"Pattern de nome arquivo não foi encontrado: {a['_pattern_nome']}"
            )
        elif 'nome' in a and "__count" not in a:
            raise RuntimeError(
                f"Nome arquivo não foi encontrado: {a['nome']}"
            )

    manifesto_dat[ARQUIVOS].sort(
        key=cmp_to_key(
            lambda arq1, arq2: _acha_ordem(arq1) - _acha_ordem(arq2)
        )
    )
    return lista_zip


def _atualizar_versoes_bases(manifesto_dat, kwargs):
    OBTER = '_obter_versao_do_json'
    versoes_bases = [
        dict(schema=a[0], versao=a[1]) for a in kwargs.pop('versoes_bases', [])
    ]
    schemas = [a[SCHEMA] for a in versoes_bases]

    for base in manifesto_dat.get(BASES_COMPATIVEIS, []):
        obter = base.get(OBTER)
        if obter:
            if obter_versao_db_scripts is None:
                raise RuntimeError('Biblioteca de scripts indisponivel')
            schema, versao = obter_versao_db_scripts(obter)
            base[SCHEMA] = schema
            base[VERSAO] = versao
            del base[OBTER]
        if base.get(SCHEMA) and base.get(SCHEMA) not in schemas:
            versoes_bases.append(base)

    if len(versoes_bases):
        manifesto_dat[BASES_COMPATIVEIS] = versoes_bases


def _obter_manifesto_do_template(pasta):
    manifesto_usado = os.path.join(pasta, MANIFESTO_SERVER)
    try:
        with open(manifesto_usado, 'r', encoding='utf-8-sig') as arq:
            manifesto_dat = json.load(arq)
            logger.info('usando ' + MANIFESTO_SERVER)
    except IOError:
        try:
            manifesto_usado = os.path.join(pasta, MANIFESTO_LOCAL)
            with open(manifesto_usado, 'r', encoding='utf-8-sig') as arq:
                manifesto_dat = json.load(arq)
                logger.info('usando ' + MANIFESTO_LOCAL)
        except IOError:
            logger.info(u'template de manifesto não encontrado. gerando...')
            manifesto_dat = DEFAULTS_PACOTE.copy()
    except Exception as e:
        logger.exception(f'Erro ao processar manifesto {manifesto_usado}')
        print('Erro ao processar: ' + manifesto_usado)
        print(e)
        raise
    return manifesto_dat


def empacotar(pasta, pasta_saida, senha, **kwargs):
    manifesto_dat = _obter_manifesto_do_template(pasta)

    # Atualizo o manifesto com o que foi passado pela linha de comando
    _atualizar_versoes_bases(manifesto_dat, kwargs)
    manifesto_dat.update(kwargs)

    arquivos = obter_arquivos(pasta, manifesto_dat, os.listdir(pasta))

    with open(os.path.join(pasta, MANIFESTO), 'w') as manifile:
        json.dump(manifesto_dat, manifile, indent=2)

    prefixo = manifesto_dat[PACOTE].replace(' ', '') + '_'
    nome_cmpkg = prefixo + \
        manifesto_dat[VERSAO].replace(' ', '').replace('.', '_') + \
        EXTENSAO_PACOTE
    saida = os.path.join(pasta_saida, nome_cmpkg)

    _excluir_com_prefixo(pasta_saida, prefixo)
    zipar(saida, arquivos=arquivos, senha=senha)
    arquivo_cmpkg = os.path.join(os.getcwd(), saida)

    # Suporte a uma pasta para os QAs
    _copiar_para_qa(nome_cmpkg, prefixo, saida)

    return arquivo_cmpkg


def _copiar_para_qa(nome_cmpkg, prefixo, saida):
    if os.environ.get('QA', '').lower() != 'true':
        return

    if os.environ.get('ALOHA', '').lower() == 'true':
        logger.info('Arquivo não será gerado na pasta de QA pois ALOHA=true')
        return

    pasta_QA = os.environ.get('PASTA_QA', PASTA_QA)
    if not os.path.exists(pasta_QA):
        logger.error(f'PASTA_QA nao encontrada: {pasta_QA}')
        return

    _excluir_com_prefixo(pasta_QA, prefixo)
    try:
        print(f'Copiando para pasta de QA: {pasta_QA}')
        shutil.copyfile(saida, os.path.join(pasta_QA, nome_cmpkg))
    except Exception:
        logger.exception(
            f'Falha ao copiar arquivo {saida} para a pasta {pasta_QA}'
        )


def zipar(destino, arquivos=None, pasta=None, senha=''):
    destino = os.path.join(os.getcwd(), destino)

    cmdline = [
        CAM_7ZA,
        'a',
        '-tzip',
        '-mx9',
        '-y'
    ]
    if senha:
        cmdline.append('-p' + senha)
    cmdline.append(destino)

    prevdir = None
    if arquivos:
        cmdline = cmdline + arquivos
    elif pasta:
        prevdir = os.getcwd()
        pasta = os.path.join(os.getcwd(), pasta)
        os.chdir(pasta)

    logger.debug('comando gerado: "{}"'.format(cmdline))
    subprocess.call(cmdline)
    if prevdir:
        os.chdir(prevdir)



# coding: utf-8
from pathlib import Path
from setuptools import setup
from versao import version_info

long_description = Path("README.rst").read_text()

setup(
    name='colibri-packaging',
    author='Colibri Agile',
    author_email='colibri.agile@gmail.com',
    platforms=['Windows'],
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html'
    version=version_info()['fileversion'],
    description='Colibri Master Packages generator',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    packages=[
        'colibri_packaging'
    ],
    package_data={
        'colibri_packaging': ['7za.exe']
    },
    data_files=[('', ['versao.py', '__version__.py', '__buildnumber__.py'])],
    classifiers=[
        'Programming Language :: Python :: 3.11'
    ],
    python_requires='>=3.11'
)
